# https://earth.bsc.es/gitlab/es/s2dverification/raw/develop_calibrate/R/Calibration.R

# Based on Vero's Calibration.R, the aim is to calibrate a forecast that has different 
# number of members than the hindcast. As input, there is:
# var_obs -->  obs[t] ,m=1         # include member dim =1 in obs?--> to decide
# var_exp -->  hindcast[t,m=mh]  
# var_fcst --> forecast[m=mf]  t=1 # one date

# anom=TRUE,  if the inputs are anomalies,  output will be anomaly 
#             (will hace to have the obs clim added to give abs value )
# anom=FALSE (defau), if the input are abs values, output will be abs value 

Calibration_fcst <- function(var_obs, var_exp, var_fcst, anom = FALSE) {
  # Ensemble inflation of a forecast following Doblas-Reyes et al. (2005)
  #if (!is.vector(var_obs)) {
  #  stop('var_obs should be a vector')
  #}
  #if (any(is.na(var_obs))|any(is.na(var_exp))) {
  #  stop('NAs are not allowed')
  #}
  
  if (length(dim(var_exp)) != 2) {
    stop('var_exp should be a matrix')
  }
  
  if (length(which(dim(var_exp) == length(var_obs))) == 0) {
    stop('var_exp should have a dimension (time) with the same length that var_obs')
  }
  
  ntime <- length(var_obs)
  nmembers<-dim(var_exp)['ensemble']
  nfmembers <- dim(var_fcst)['ensemble']  # but then the iput has to have a named dim ensemble
  # take 1 dim
  var_fcst <- var_fcst[1,]

  #if (!all(dim(var_exp) == c(nmembers, ntime))) {
  #  var_exp <- t(var_exp)
  #}
  if (all(names(dim(var_exp))!=c('ensemble','sdate'))) {
    var_exp <- t(var_exp)
  } 
  
  
  if (!anom) {
    #climObs <- NA * var_obs
    #climPred <- NA * var_obs
    #for (t in 1:length(var_obs))
    #{
    climObs <- mean(var_obs)
    climPred <- mean(var_exp)
    #}
    var_obs <- var_obs - climObs  # reassign var_obs with anomaly values
    for (i in 1:nmembers) {
      var_exp[i, ] <- var_exp[i, ] - climPred # reassign var_exp with anomaly values
    }
    # Compute anomaly of forecast
    var_fcst <- var_fcst - climPred # reassign var_fcst with anomaly values
    
  }

  
  calibrated <- NA * var_fcst
  
  #for (t in 1:ntime) {
    # defining forecast,hindcast and observation in cross-validation
    #fcst <- var_exp[, t]
    #hcst <- var_exp[, -t]
    #obs <- var_obs[-t]
    
  fcst <- var_fcst
  hcst <- var_exp
  obs <- var_obs


  #coefficients
  em_fcst <- mean(fcst)
  em_hcst <- apply(hcst, c(2), mean)
  corr<- cor(em_hcst, obs)
  sd_obs <- sd(obs)
  sd_em_hcst <- sd(em_hcst)

  fcst_diff <- fcst - em_fcst
  hcst_diff <- NA * hcst
  for (n in 1:nmembers) {
  hcst_diff[n, ] <- hcst[n, ] - em_hcst
  }
  sd_hcst_diff <- sd(hcst_diff)

  a <- corr * (sd_obs / sd_em_hcst)
  b <- (sd_obs / sd_hcst_diff) * sqrt(1 - (corr ^ 2))

  # calibrated output
  calibrated <- (a * em_fcst) + (b * fcst_diff)

  if (!anom) {
      calibrated <- calibrated + climObs
  }
  
  
  #}
  
  return(calibrated)
  
}

# version that admits NAs (done for the 1st attempst with 2017 data)
Calibration_fcst_NA <- function(var_obs, var_exp, var_fcst, anom = FALSE) {
  # Ensemble inflation of a forecast following Doblas-Reyes et al. (2005)
  #if (!is.vector(var_obs)) {
  #  stop('var_obs should be a vector')
  #}
  #if (any(is.na(var_obs))|any(is.na(var_exp))) {
  #  stop('NAs are not allowed')
  #}
  
  if (length(dim(var_exp)) != 2) {
    stop('var_exp should be a matrix')
  }
  
  if (length(which(dim(var_exp) == length(var_obs))) == 0) {
    stop('var_exp should have a dimension (time) with the same length that var_obs')
  }
  
  ntime <- length(var_obs)
  nmembers<-dim(var_exp)['ensemble']
  nfmembers <- dim(var_fcst)['ensemble']  # but then the iput has to have a named dim ensemble
  # take 1 dim
  var_fcst <- var_fcst[1,]

  #if (!all(dim(var_exp) == c(nmembers, ntime))) {
  #  var_exp <- t(var_exp)
  #}
  if (all(names(dim(var_exp))!=c('ensemble','sdate'))) {
    var_exp <- t(var_exp)
  } 
  
  
  
  if (!anom) {
    #climObs <- NA * var_obs
    #climPred <- NA * var_obs
    #for (t in 1:length(var_obs))
    #{
    climObs <- mean(var_obs, na.rm=TRUE)
    climPred <- mean(var_exp)
    #}
    var_obs <- var_obs - climObs  # reassign var_obs with anomaly values
    for (i in 1:nmembers) {
      var_exp[i, ] <- var_exp[i, ] - climPred # reassign var_exp with anomaly values
    }
    # Compute anomaly of forecast
    var_fcst <- var_fcst - climPred # reassign var_fcst with anomaly values
    
  }

  
  calibrated <- NA * var_fcst
  
  #for (t in 1:ntime) {
    # defining forecast,hindcast and observation in cross-validation
    #fcst <- var_exp[, t]
    #hcst <- var_exp[, -t]
    #obs <- var_obs[-t]
    
  fcst <- var_fcst
  hcst <- var_exp
  obs <- var_obs


  #coefficients
  em_fcst <- mean(fcst)
  em_hcst <- apply(hcst, c(2), mean)
 # to make it work with NA in obs 
  if(any(is.na(obs))){
  nona_em_hcst<-em_hcst[-which(is.na(obs))]
  nona_obs<-obs[-which(is.na(obs))]
  corr<- cor(nona_em_hcst, nona_obs)
  }else{
  corr <- cor(em_hcst, obs)
  }
  sd_obs <- sd(obs, na.rm=TRUE)
  sd_em_hcst <- sd(em_hcst)

  fcst_diff <- fcst - em_fcst
  hcst_diff <- NA * hcst
  for (n in 1:nmembers) {
  hcst_diff[n, ] <- hcst[n, ] - em_hcst
  }
  sd_hcst_diff <- sd(hcst_diff)

  a <- corr * (sd_obs / sd_em_hcst)
  b <- (sd_obs / sd_hcst_diff) * sqrt(1 - (corr ^ 2))

  # calibrated output
  calibrated <- (a * em_fcst) + (b * fcst_diff)

  if (!anom) {
      calibrated <- calibrated + climObs
  }
  
  
  #}
  
  return(calibrated)
  
}

# Original function from Vero, use to calibrate hindcast 
#to obtained calibrated terciles 
Calibration <- function(var_obs, var_exp, anom = FALSE) {
  # Ensemble inflation of a forecast following Doblas-Reyes et al. (2005)
  #if (!is.vector(var_obs)) {
  #  stop('var_obs should be a vector')
  #}
  #if (any(is.na(var_obs))|any(is.na(var_exp))) {
  #  stop('NAs are not allowed')
  #}
  
  if (length(dim(var_exp)) != 2) {
    stop('var_exp should be a matrix')
  }
  
  if (length(which(dim(var_exp) == length(var_obs))) == 0) {
    stop('var_exp should have a dimension (time) with the same length that var_obs')
  }
  
  ntime <- length(var_obs)
  nmembers<-dim(var_exp)['ensemble']
  
  #if (!all(dim(var_exp) == c(nmembers, ntime))) {
  #  var_exp <- t(var_exp)
  #}
  if (all(names(dim(var_exp))!=c('ensemble','sdate'))) {
    var_exp <- t(var_exp)
  } 
  
  if (!anom) {
    climObs <- NA * var_obs
    climPred <- NA * var_obs
    for (t in 1:length(var_obs))
    {
      climObs[t] <- mean(var_obs)
      climPred[t] <- mean(var_exp[, -t])
    }
    var_obs <- var_obs - climObs
    for (i in 1:nmembers) {
      var_exp[i, ] <- var_exp[i, ] - climPred
    }
  }
  
  calibrated <- NA * var_exp
  
  for (t in 1:ntime) {
    # defining forecast,hindcast and observation in cross-validation
    fcst <- var_exp[, t]
    hcst <- var_exp[, -t]
    obs <- var_obs[-t]
    
    #coefficients
    em_fcst <- mean(fcst)
    em_hcst <- apply(hcst, c(2), mean)
    corr <- cor(em_hcst, obs)
    sd_obs <- sd(obs)
    sd_em_hcst <- sd(em_hcst)
    
    fcst_diff <- fcst - em_fcst
    hcst_diff <- NA * hcst
    for (n in 1:nmembers) {
      hcst_diff[n, ] <- hcst[n, ] - em_hcst
    }
    sd_hcst_diff <- sd(hcst_diff)
    
    a <- corr * (sd_obs / sd_em_hcst)
    b <- (sd_obs / sd_hcst_diff) * sqrt(1 - (corr ^ 2))
    
    # calibrated output
    calibrated[, t] <- (a * em_fcst) + (b * fcst_diff)
    
    if (!anom) {
      calibrated[, t] <- calibrated[, t] + climObs[t]
    }
  }
  
  return(calibrated)
  
}

