library(easyNCDF)

save_obs_country_file <-
  function(variable,
           obs,
           fcst.sdate,
           outfile,
           fcst.type,
           monthnames) {

    if (fcst.type == 'seasonal' || variable == 'psl'){
    load('/esarchive/oper/S2S4E-backend-BSC/data-analysis/masks/mask_europe_system5_2.Rdata')
    } else {
    load('/esarchive/oper/S2S4E-backend-BSC/data-analysis/masks/mask_europe_ncep_2.Rdata')
    }
    
    ## TODO: reorder calibrated values array (if needed)
    obs <- aperm(obs, c(2, 1, 3))
    names(dim(obs)) <- c('latitude', 'longitude','time')

    obs.country <- array(NA, dim=c(length(europe.countries.iso), dim(obs)[3]))
    for (country in 1:length(europe.countries.iso)){
      Country_mask <- mask[country,,]
      Country_mask [Country_mask == 0 ] <- NA
      for (t in 1:dim(obs)[3]){
	    obs.country[country,t] <- mean(obs[,,t]*Country_mask, na.rm=TRUE)
          }
    }

    if (variable == "prlr") {
      units = c('mm')
      variable.longname = 'Total precipitation'
      if (fcst.type == 'seasonal'){
        for (month in 1:length(monthnames)){
            month.sdate <- paste0(substr(monthnames[month],1,7), "-01")
            month.lastday <- as.integer(substr((seq(as.Date(month.sdate, "%Y-%m-%d"),
                           length=2,by="months")-1)[2], 9, 10))
            obs.country[,month] <- obs.country[,month]*month.lastday*86400000      
        }
      } else {
        obs.country <- obs.country * 1000 * 3600 * 24 * 7      
      }
    } else if (variable == "tasmin") {
      units = c('ºC')
      variable.longname = 'Minimum daily surface temperature' 
      obs.country <- obs.country - 273
    } else if (variable == "tasmax") {
      units = c('ºC')
      variable.longname = 'Maximum daily surface temperature' 
      obs.country <- obs.country - 273
    } else if (variable == "tas") {
      units = c('ºC')
      variable.longname = 'Minimum daily temperature' 
      obs.country <- obs.country - 273
    } else if (variable == "sfcWind") {
      units = c('m/s')
      variable.longname = 'Surface Wind Speed' 
    } else if (variable == "rsds") {
      units = c('W/m2')
      variable.longname = 'Long wave solar radiation downwards' 
    } else if (variable == "psl") {
      units = c('Pa')
      variable.longname = 'Mean sea level pressure' 
      obs.country <- obs.country / 100
    } else {
      units = c('')
      variable.longname = ''
    }
 

    metadata <- list(obs.country = 
        list(
            name = paste0(variable, "_country"),
            standard_name = paste0(variable.longname, " (Country-Aggregated)"),
            units = units
        )
    )
    
    attr(obs.country, 'variables') <- metadata
    names(dim(obs.country)) <- c('Country', 'time')
    
    if (fcst.type == 'seasonal') {
      time <- 1:12
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          standard_name = 
            monthnames
          ,
          units = paste0('months since ', fcst.sdate, ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'
    }
    
    if (fcst.type == 'seasonal') {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'months since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'Time step'
    } else {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'months since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'time_step'
      
    }
    
    country <- 1:length(europe.countries.iso)
    dim(country) <- length(country)
    metadata <- list( country = list(
                                standard_name = paste(europe.countries.iso, collapse=" "),
		                units = 'Country ISO 3166-1 alpha 3 Code'))
    attr(country, 'variables') <- metadata
    names(dim(country)) <- 'Country'


    ArrayToNc(
      list(
        country,
        time,
        time_step,
        obs.country
      ),
     outfile
    )
 
 }
save_ensemble_country_file <-
  function(variable,
           calibrated_fcst.country,
           fcst.sdate,
           outfile,
           monthnames,
           europe.countries.iso,
           fcst.type) {

    if (variable == "prlr") {
      units = c('mm')
      variable.longname = 'Total precipitation'
      if (fcst.type == 'seasonal'){
        for (month in 1:length(monthnames)){

            month.sdate <- paste0(substr(monthnames[month],1,7), "-01")
            month.lastday <- as.integer(substr((seq(as.Date(month.sdate, "%Y-%m-%d"),
                           length=2,by="months")-1)[2], 9, 10))
            calibrated_fcst.country[,,month] <- calibrated_fcst.country[,,month]*month.lastday*86400000      
        }
      } else {
        calibrated_fcst.country <- calibrated_fcst.country * 1000 * 3600 * 24 * 7      
      }
    } else if (variable == "tasmin") {
      units = c('ºC')
      variable.longname = 'Minimum daily surface temperature' 
      calibrated_fcst.country <- calibrated_fcst.country - 273
    } else if (variable == "tasmax") {
      units = c('ºC')
      variable.longname = 'Maximum daily surface temperature' 
      calibrated_fcst.country <- calibrated_fcst.country - 273
    } else if (variable == "tas") {
      units = c('ºC')
      variable.longname = 'Minimum daily temperature' 
      calibrated_fcst.country <- calibrated_fcst.country - 273
    } else if (variable == "sfcWind") {
      units = c('m/s')
      variable.longname = 'Surface Wind Speed' 
    } else if (variable == "rsds") {
      units = c('W/m2')
      variable.longname = 'Long wave solar radiation downwards' 
    } else if (variable == "psl") {
      units = c('Pa')
      variable.longname = 'Mean sea level pressure' 
      calibrated_fcst.country <- calibrated_fcst.country / 100
    } else {
      units = c('')
      variable.longname = ''
    }
 

    metadata <- list(calibrated_fcst.country = 
        list(
            name = paste0(variable, "_country"),
            standard_name = paste0(variable.longname, " (Country-Aggregated)"),
            units = units
        )
    )
    
    attr(calibrated_fcst.country, 'variables') <- metadata
    names(dim(calibrated_fcst.country)) <- c('Country', 'ensemble', 'time')
    
    if (fcst.type == 'seasonal') {
      time <- 1:3
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          standard_name = paste(
            monthnames[1],
            ', ',
            monthnames[2]
            ,
            ', ',
            monthnames[3],
            sep = ''
          ),
          units = paste0('months since ', fcst.sdate, ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'
    } else {
      time <- 1:4
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          units = paste0('weeks since ', paste(substr(fcst.sdate,1,4),substr(fcst.sdate,5,6),substr(fcst.sdate,7,8), sep='-'), ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'

    }

    
    if (fcst.type == 'seasonal') {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'months since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'Time step'
    } else {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'months since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'time_step'
      
    }
    
    country <- 1:length(europe.countries.iso)
    dim(country) <- length(country)
    metadata <- list( country = list(
                                standard_name = paste(europe.countries.iso, collapse=" "),
		                units = 'Country ISO 3166-1 alpha 3 Code'))
    attr(country, 'variables') <- metadata
    names(dim(country)) <- 'Country'


    ArrayToNc(
      list(
        country,
        time,
        time_step,
        calibrated_fcst.country
      ),
     outfile
    )
 
 }

save_probs_country_file <-
  function(variable,
           tercile_probs,
           extreme_probs,
           fcst.sdate,
           outfile,
           monthnames,
           europe.countries.iso,
           fcst.type) {
    
    prob_bn.country <- tercile_probs[1, , ]
    prob_n.country <- tercile_probs[2, , ]
    prob_an.country <- tercile_probs[3, , ]
    prob_bp10.country  <- extreme_probs[1, , ]
    prob_ap90.country <- extreme_probs[3, , ]
     
    metadata <- list(
      prob_bn = list(
        name = 'prob_bn',
        standard_name = paste(
          'Probability below normal category (Country-Aggregated)', sep = '')
      ),
      prob_n = list(
        name = 'prob_n',
        standard_name = paste('Probability normal category (Country-Aggregated)', sep ='')
      ),
      prob_an = list(
        name = 'prob_an',
        standard_name = paste(
          'Probability above normal category (Country-Aggregated)', sep = '')
      ),
      prob_bp10 = list(
        name = 'prob_bp10',
        standard_name = paste(
          'Probability below extreme category (Country-Aggregated)', sep = '')
      ),
      prob_ap90 = list(
        name = 'prob_ap90',
        standard_name = paste(
          'Probability above extreme category (Country-Aggregated)', sep = '')
      ))

    attr(prob_bn.country, 'variables') <- metadata[1]
    attr(prob_n.country, 'variables') <- metadata[2]
    attr(prob_an.country, 'variables') <- metadata[3]
    attr(prob_bp10.country, 'variables') <- metadata[4]
    attr(prob_ap90.country, 'variables') <- metadata[5]

    names(dim(prob_bn.country)) <- c('Country', 'time')
    names(dim(prob_n.country)) <- c('Country', 'time')
    names(dim(prob_an.country)) <- c('Country', 'time')
    names(dim(prob_bp10.country)) <- c('Country', 'time')
    names(dim(prob_ap90.country)) <- c('Country', 'time')
 
    if (fcst.type == 'seasonal') {
      time <- 1:3
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          standard_name = paste(
            monthnames[1],
            ', ',
            monthnames[2]
            ,
            ', ',
            monthnames[3],
            sep = ''
          ),
          units = paste0('months since ', fcst.sdate, ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'
    } else {
      time <- 1:4
      dim(time) <- length(time)
      metadata <- 
        list(time = list(
          units = paste0('weeks since ', paste(substr(fcst.sdate,1,4),substr(fcst.sdate,5,6),substr(fcst.sdate,7,8), sep='-'), ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'

    }

    if (fcst.type == 'seasonal') {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'months since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'Time step'
    } else {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'weeks since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'time_step'

     }
    
    
    country <- 1:length(europe.countries.iso)
    dim(country) <- length(country)
    metadata <- list( country = list(
                                standard_name = paste(europe.countries.iso, collapse=" "),
		                units = 'Country ISO 3166-1 alpha 3 Code'))
    attr(country, 'variables') <- metadata
    names(dim(country)) <- 'Country'


    ArrayToNc(
      list(
        country,
        time, time_step,
        prob_bn.country,
        prob_n.country,
        prob_an.country,
        prob_bp10.country,
        prob_ap90.country
      ),
     outfile
    )
 
 }

save_skill_country_file <-
  function(variable,
           rpss.country,
           bss10.country,
           bss90.country,
           fcst.sdate,
           outfile,
           monthnames,
           europe.countries.iso,
           fcst.type) {

    metadata <- list(
      rpss.country = list(
        name = 'rpss',
        standard_name = paste(
          'Country-Aggregated Terciles skill score', sep = '')
      ),
      bss10.country = list(
        name = 'bsp10',
        standard_name = paste('Country-Aggregated Lower extreme tercile skill score', sep ='')
      ),
      bss90.country = list(
        name = 'bsp90',
        standard_name = paste(
          'Country-Aggregated Upper extreme tercile skill score', sep = '')
      ))

    attr(rpss.country, 'variables') <- metadata[1]
    attr(bss10.country, 'variables') <- metadata[2]
    attr(bss90.country, 'variables') <- metadata[3]

    names(dim(rpss.country)) <- c('Country', 'time')
    names(dim(bss10.country)) <- c('Country', 'time')
    names(dim(bss90.country)) <- c('Country', 'time')
 
    if (fcst.type == 'seasonal') {
      time <- 1:3
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          standard_name = paste(
            monthnames[1],
            ', ',
            monthnames[2]
            ,
            ', ',
            monthnames[3],
            sep = ''
          ),
          units = paste0('months since ', fcst.sdate, ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'
    } else {
      time <- 1:4
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          units = paste0('weeks since ', fcst.sdate, ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'


    }
    

    if (fcst.type == 'seasonal') {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'months since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'Time step'
    } else {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'weeks since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'time_step'
    }

   
    country <- 1:length(europe.countries.iso)
    dim(country) <- length(country)
    metadata <- list( country = list(
                                standard_name = paste(europe.countries.iso, collapse=" "),
		                units = 'Country ISO 3166-1 alpha 3 Code'))
    attr(country, 'variables') <- metadata
    names(dim(country)) <- 'Country'


    ArrayToNc(
      list(
	country,
        time,
        rpss.country,
	bss10.country,
        bss90.country,
        time_step
       ),
     outfile
    )
 
 }


save_terciles_country_file <-

  function(variable,
           terciles_hcst,
           extremes_hcst,
           fcst.sdate,
           outfile,
           monthnames,
           europe.countries.iso,
           fcst.type) {

    p33.country <- terciles_hcst[1, ,]
    p66.country <- terciles_hcst[2, ,]
    p10.country <- extremes_hcst[1, ,]
    p90.country <- extremes_hcst[2, ,]

    if (variable == "prlr") {
      if (fcst.type == 'seasonal'){
        for (month in 1:(dim(p33.country)[2])){

          month.sdate <- paste0(substr(monthnames[month],1,7), "-01")
          month.lastday <- as.integer(substr((seq(as.Date(month.sdate, "%Y-%m-%d"),
                           length=2,by="months")-1)[2], 9, 10))
          p33.country[,month] <- p33.country[,month]*month.lastday*86400000
          p66.country[,month] <- p66.country[,month]*month.lastday*86400000
          p90.country[,month] <- p90.country[,month]*month.lastday*86400000
          p10.country[,month] <- p10.country[,month]*month.lastday*86400000
        }
      } else {
        p33.country <- p33.country * 1000 * 3600 * 24 * 7
        p66.country <- p66.country * 1000 * 3600 * 24 * 7
        p90.country <- p90.country * 1000 * 3600 * 24 * 7
        p10.country <- p10.country * 1000 * 3600 * 24 * 7
      }
    } else if (variable == "tasmin" || variable == "tasmax" || variable == "tas") {
      p33.country <- p33.country - 273
      p66.country <- p66.country - 273
      p10.country <- p10.country - 273
      p90.country <- p90.country - 273
    } else if (variable == "psl") {
      p33.country <- p33.country / 100
      p66.country <- p66.country / 100
      p10.country <- p10.country / 100
      p90.country <- p90.country / 100
    } else {
      warning("NO UNITS CONVERSION")
    }


    metadata <- list(
      p33.country = list(
        name = 'p33',
        standard_name = paste(
          'Country-aggregated Lower tercile', sep = '')
      ),
      p66.country = list(
        name = 'p66',
        standard_name = paste('Country-aggregated Upper tercile', sep ='')
      ),
      p10.country = list(
        name = 'p10',
        standard_name = paste(
          'Country-aggregated Lower extreme tercile', sep = '')
      ),
      p90.country = list(
        name = 'p90',
        standard_name = paste(
          'Country-aggregated Upper extreme tercile', sep = '')
      ))

    attr(p33.country, 'variables') <- metadata[1]
    attr(p66.country, 'variables') <- metadata[2]
    attr(p10.country, 'variables') <- metadata[3]
    attr(p90.country, 'variables') <- metadata[4]

    names(dim(p33.country)) <- c('Country', 'time')
    names(dim(p66.country)) <- c('Country', 'time')
    names(dim(p10.country)) <- c('Country', 'time')
    names(dim(p90.country)) <- c('Country', 'time')
 
    if (fcst.type == 'seasonal') {
      time <- 1:3
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          standard_name = paste(
            monthnames[1],
            ', ',
            monthnames[2]
            ,
            ', ',
            monthnames[3],
            sep = ''
          ),
          units = paste0('months since ', fcst.sdate, ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'
    } else {
      time <- 1:4
      dim(time) <- length(time)
      metadata <-
        list(time = list(
          units = paste0('weeks since ', fcst.sdate, ' 00:00:00')
        ))
      attr(time, 'variables') <- metadata
      names(dim(time)) <- 'time'
    }
    
    
    if (fcst.type == 'seasonal') {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'months since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'Time step'
    } else {
      time_step <- 1
      dim(time_step) <- length(time_step)
      metadata <-
        list(time_step = list(unitss = paste0(
          'weeks since ', fcst.sdate, ' 00:00:00'
        )))
      attr(time_step, 'variables') <- metadata
      names(dim(time_step)) <- 'time_step'
      
    }
    
    country <- 1:dim(p33.country)['Country']
    dim(country) <- length(country)
    metadata <- list( country = list(
                                standard_name = paste(europe.countries.iso, collapse=" "),
		                units = 'Country ISO 3166-1 alpha 3 Code'))
    attr(country, 'variables') <- metadata
    names(dim(country)) <- 'Country'

    ArrayToNc(
      list(
        country,
        time,
        time_step,
        p33.country,
        p66.country,
        p10.country,
        p90.country),
        outfile
    )
 
 }
