
library(easyVerification)
library(multiApply)

Compute_skill <- function(calibrated_hcst2, sub_obs2, ncores=1, split_factor=1, na.rm=FALSE) {
 
    veriUnwrap <- easyVerification:::veriUnwrap
    fRPSS<-Apply(data=list(calibrated_hcst2, sub_obs2),
               target_dims=list(c('sdate','member'),c('sdate','member')),
               fun="veriApply",
               verifun="FairRpss",
               prob=c(1/3,2/3),
               ensdim=2,
               ncores=ncores,
               split_factor=split_factor,
               tdim=1,
               na.rm=na.rm)[[1]] * 100

    print("#################-RPSS-###########################") 
    BSS10<-Apply(data=list(calibrated_hcst2, sub_obs2),
               target_dims=list(c('sdate','member'),c('sdate','member')),
              fun="veriApply",
               verifun="FairRpss",
               prob=(1/10),
               ensdim=2,
               tdim=1,
               ncores=ncores,
               split_factor=split_factor,
               na.rm=na.rm)[[1]] * 100
  
    print("#################-BSS10-###########################") 
    BSS90<-Apply(data=list(calibrated_hcst2, sub_obs2),
               target_dims=list(c('sdate','member'),c('sdate','member')),
               fun="veriApply",
               verifun="FairRpss",
               prob=(9/10),
               ensdim=2,
               tdim=1,
               ncores=ncores,
               split_factor=split_factor,
               na.rm=na.rm)[[1]] * 100

     print("#################-BSS90-###########################") 
     fRPSS <- Subset(fRPSS,c('member'),list(1),drop='selected')
     BSS10 <- Subset(BSS10,c('member'),list(1),drop='selected')
     BSS90 <- Subset(BSS90,c('member'),list(1),drop='selected')

     fRPSS[!is.finite(fRPSS)] <- NA
     BSS10[!is.finite(BSS10)] <- NA
     BSS90[!is.finite(BSS90)] <- NA
    
     return(list(fRPSS = fRPSS, BSS10 = BSS10, BSS90 = BSS90))

}

Compute_probs <- function(calibrated_hcst, terciles_hcst, extremes_hcst, ncores=1, split_factor=1, na.rm=FALSE) {

  if (na.rm == FALSE) {
    c2p <- function(x, t) {
    colMeans(convert2prob(as.vector(x), threshold = as.vector(t)))
    }
  } else {
    c2p <- function(x, t) {
      if (any(!is.na(x))){
        colMeans(convert2prob(as.vector(x), threshold = as.vector(t)))
      } else {
        c(NA,NA, NA)
      }
    }
  }
  
  tercile_probs_hcst <- Apply(
                          data = list(x = calibrated_hcst, t = terciles_hcst),
                          target_dims = list("member", "bin"),
                          c2p,
                          output_dims = "bin",
                          split_factor = split_factor,
                          ncores = ncores
                          )[[1]] * 100

  extreme_probs_hcst <- Apply(
                            data = list(x = calibrated_hcst, t = extremes_hcst),
                            target_dims = list("member", "bin"),
                            c2p,
                            output_dims = "bin",
                            split_factor = split_factor,
                            ncores = ncores
                            )[[1]] * 100

  probs <- list(tercile_probs_hcst = tercile_probs_hcst, extreme_probs_hcst = extreme_probs_hcst)
  
  return(probs)

}

Country_mean <- function(pointdata, mask.path) {
   
  load(mask.path)
  data.country <- array(NA, dim=c(length(europe.countries.iso)))
  for (country in 1:length(europe.countries.iso)){
      Country_mask <- mask[country,,]
      Country_mask [Country_mask == 0 ] <- NA
      data.country[country] <- mean(pointdata*Country_mask, na.rm=TRUE)
  }
  return(data.country)
}
